function [] = export_material(File, export_filename,output,KPI,txt_out,txt_fractions,txt_KPI,txt_type)
%SUMMARY The function writes the material information to the scenario file.
%   The location where to write some data is dynamic, which means that it
%   is dependent on the amount of fractions and waste types defined in
%   excel. These locations are therefore generated automatically in Excel
%   (in the output tab) and are read by excel. This type of data will start
%   with "txt". The other writing commands are static and their location is
%   decided in the code below.



Excel = actxserver ('Excel.Application');
File = [File,'\',export_filename,];

if ~exist(File,'file')
    ExcelWorkbook = Excel.workbooks.Add;
    ExcelWorkbook.SaveAs(File,1);
    ExcelWorkbook.Close(false);
end
invoke(Excel.Workbooks,'Open',File);

%% Output tab

disp('writing material layer started');

xlswrite1(export_filename,cellstr(txt_type'),'Output','L3');
xlswrite1(export_filename,cellstr('Quantification of all flows (t/yr) and TC'),'Output','H2');


for i=1:size(output,2)
    
    xlswrite1(export_filename,cellstr('Flow'),'Output',char(txt_out(i,1)));
    xlswrite1(export_filename,txt_fractions,'Output',char(txt_out(i,2)));
    xlswrite1(export_filename,output(i).type,'Output',char(txt_out(i,3)));
    xlswrite1(export_filename,output(i).from,'Output',char(txt_out(i,4)));
    xlswrite1(export_filename,output(i).to,'Output',char(txt_out(i,5)));
    xlswrite1(export_filename,output(i).value,'Output',char(txt_out(i,6)));
    
end

%% Results tab


% flow table overview 
xlswrite1(export_filename,cellstr('Total (t/yr)'),'Results','D4');
xlswrite1(export_filename,cellstr(txt_type'),'Results','E4');
%generated
xlswrite1(export_filename,cellstr(txt_fractions),'Results',char(txt_KPI(1,2)));
xlswrite1(export_filename,cellstr(KPI(1).name),'Results',char(txt_KPI(1,1)));
xlswrite1(export_filename,KPI(1).fractions,'Results',char(txt_KPI(1,3)));
xlswrite1(export_filename,KPI(1).matrix,'Results',char(txt_KPI(1,4)));
%sorted
xlswrite1(export_filename,cellstr(txt_fractions),'Results',char(txt_KPI(2,2)));
xlswrite1(export_filename,cellstr(KPI(3).name),'Results',char(txt_KPI(2,1)));
xlswrite1(export_filename,KPI(3).fractions,'Results',char(txt_KPI(2,3)));
xlswrite1(export_filename,KPI(3).matrix,'Results',char(txt_KPI(2,4)));
%recycled
xlswrite1(export_filename,cellstr(txt_fractions),'Results',char(txt_KPI(3,2)));
xlswrite1(export_filename,cellstr(KPI(4).name),'Results',char(txt_KPI(3,1)));
xlswrite1(export_filename,KPI(4).fractions,'Results',char(txt_KPI(3,3)));
xlswrite1(export_filename,KPI(4).matrix,'Results',char(txt_KPI(3,4)));
%company
xlswrite1(export_filename,cellstr(txt_fractions),'Results',char(txt_KPI(4,2)));
xlswrite1(export_filename,cellstr(KPI(5).name),'Results',char(txt_KPI(4,1)));
xlswrite1(export_filename,KPI(5).fractions,'Results',char(txt_KPI(4,3)));
xlswrite1(export_filename,KPI(5).matrix,'Results',char(txt_KPI(4,4)));

%efficiency overview
xlswrite1(export_filename,cellstr('Aggregated'),'Results','M5');
xlswrite1(export_filename,cellstr(txt_fractions),'Results','M6');
%collections efficiency
xlswrite1(export_filename,cellstr('Collection efficiency'),'Results','N4');
xlswrite1(export_filename,KPI(2).eff_fr,'Results','N6');
xlswrite1(export_filename,KPI(2).eff_tot,'Results','N5');
%sorting efficiency
xlswrite1(export_filename,cellstr('Sorting efficiency'),'Results','O4');
xlswrite1(export_filename,KPI(3).eff_fr,'Results','O6');
xlswrite1(export_filename,KPI(3).eff_tot,'Results','O5');
%recycling efficiency
xlswrite1(export_filename,cellstr('Material recycling'),'Results','P4');
xlswrite1(export_filename,KPI(4).eff_fr,'Results','P6');
xlswrite1(export_filename,KPI(4).eff_tot,'Results','P5');
%company efficiency
xlswrite1(export_filename,cellstr('Company recycling'),'Results','Q4');
xlswrite1(export_filename,KPI(5).eff_tot,'Results','Q5');
%%

invoke(Excel.ActiveWorkbook,'Save');
Excel.Quit
Excel.delete
clear Excel

disp('writing material layer done');

end

