clear all;
tic % tic & toc commands time the duration of program
disp('--- program started ---');

%--- DEFINE filename ---
filename = 'Model_matlab_ReMidt.xlsx'; %Model_matlab_COMPANY


% make export filename based o0n the chosen scenario
path = pwd; %gets the path of the current folder
year = xlsread(filename,'Data','C3');
[~,scenario,~] = xlsread(filename,'Data','D3'); 
export_filename = char(scenario);
message = ['running: ',export_filename, ' ',num2str(year) ];
disp(message);
export_filename = [export_filename,'_',num2str(year),'.xlsx'];

%% Material Layer

%--- READ DATA-material --- 
%this function reads all the data used in the program (except the data used in sensitivity) 
[list_X,list_T,Data_X,Data_T,dest_fr,inhab,amount_fractions,amount_flows,process_category,bio_tr,rest_index,txt_out,txt_KPI,company_level,km,E_tr,E_pr,txt_fuels,txt_pr_E,amount_pr,cal_val,incineration,bio_energy,txt_emission,txt_fractions,txt_type,txt_flows,txt_incinerated,txt_AD] = read_data(filename);
%
%--- HANDLE DATA ---   
% This function gives the "model" structure, all the data is now gathered
% in one structure
[model,sortedtech] = handle_data(list_X,list_T,Data_X,Data_T,amount_fractions,inhab);

%
%--- MFA algoritm ---
% This function calculates the missing flows and present it in the
% structure "model_adj".
[model_adj, output] = MFA_algoritm(model,amount_flows,sortedtech);

%
%--- Material-INDICATORS ---
% This function caluclates the indicators for the material layer and
% returns them in the "KPI" structure
[KPI] = material_indicators( model_adj,process_category,bio_tr,rest_index, amount_fractions,amount_flows,dest_fr,company_level);

%
%--- XLSWRITE-material ---
% This function exports the material data to excel in the output and results tab
export_material(path,export_filename,output,KPI,txt_out,txt_fractions,txt_KPI,txt_type);
toc

%% Energy layer

%--- Energy ---
% This function gives save the Feedstock-, Process-, Transport- and Produced
% amount of energy in the energy struct
[energy] = Energy(output,km,E_tr,E_pr,amount_flows,txt_fuels,txt_pr_E,amount_pr,cal_val,incineration,bio_energy,process_category);%output out is not necessary for the moment

%
%--- Energy indicators --- 
%Energy efficiency is added to the KPI struct, the energy struct created
%above is also saved in this "KPI" struct.
[KPI] = energy_indicators( KPI,amount_fractions,amount_flows,energy);

%
%--- XLSWRITE-energy ---
% This function exports the energy data to excel in the energy,emissions and result tab
export_energy(path,export_filename,KPI,txt_KPI,energy,txt_emission,txt_fractions,txt_type,amount_pr,txt_flows,txt_incinerated,txt_AD);
toc

%% Sensitivy analysis

%This fucntion reads the sensitivity info from excel, calculates the sensitivity of the model by changing the
%certain TC and running the program again and writes the results to excel in the sensitity tab of your scenario file
%The results are saved in the sensitiv struct. If you want more detailed information of the restults, 
%the KPI struct of each iteration is saved in the PI_sens struct. 
[sensitiv,PI_sens] = sensitivity(path,filename,export_filename,list_X,list_T,Data_X,Data_T,amount_fractions,inhab,amount_flows,process_category,bio_tr,rest_index,dest_fr,KPI,company_level,km,E_tr,E_pr,txt_fuels,txt_pr_E,amount_pr,cal_val,incineration,bio_energy);


