function [sensitiv,PI_sens] = sensitivity(File, filename,export_filename,list_X,list_T,Data_X,Data_T,amount_fractions,inhab,amount_flows,process_category,bio_tr,rest_index,dest_fr,KPI,company_level,km,E_tr,E_pr,txt_fuels,txt_pr_E,amount_pr,cal_val,incineration,bio_energy)
%SUMMARY The function reads the sensitivity info, calculates the results
%and writes it to your scenario file.

disp('sensitivity started');
%   Read info
[~,txt,~] = xlsread(filename,'Flows','I10');
[sens,text,~] = xlsread(filename,'Sensitivity',char(txt));
sens( :, all( isnan(sens), 1 ) ) = []; %this delete NaN columns
sens(any(isnan(sens),2),:)=[]; %this delete rows

if ~isempty(sens)
%Loop through sens => calculation
for k= 1:size(sens)
Data_T_adj = Data_T;

% increase the parameters
for i=1:size(Data_T_adj)
    if Data_T_adj(i,2)== sens(k,1)%change the first 1
        if sens(k,4)==0 %change all waste types
            Data_T_adj(i+sens(k,3),(3:(2+amount_flows)))= Data_T_adj(i+sens(k,3),3:(2+amount_flows)).*(1+sens(k,5));
        else %change specific waste type
            Data_T_adj(i+sens(k,3),2+sens(k,4))= Data_T_adj(i+sens(k,3),2+sens(k,4)).*(1+sens(k,5));
        end
        increase = i;
        break;       
    end
        
end
%decrease the parameters if necessary
for i=1:size(Data_T_adj)
    if sens(k,2) == 0
        break
    else
        if Data_T_adj(i,2)== sens(k,2)
            if sens(k,4)==0 %change all waste types
                Data_T_adj(i+sens(k,3),3:(2+amount_flows))= (100-Data_T_adj(increase+sens(k,3),3:(2+amount_flows)));
            else %change specific waste type
                Data_T_adj(i+sens(k,3),2+sens(k,4))= (100-Data_T_adj(increase+sens(k,3),2+sens(k,4)));
            end
            break;
        end
    end   
end

%run entire program again
[model_sens_empty,sortedtech] = handle_data(list_X,list_T,Data_X,Data_T_adj,amount_fractions,inhab);
[model_sens_full, output] = MFA_algoritm(model_sens_empty,amount_flows,sortedtech);
[energy] = Energy(output,km,E_tr,E_pr,amount_flows,txt_fuels,txt_pr_E,amount_pr,cal_val,incineration,bio_energy,process_category);
[PI] = material_indicators( model_sens_full,process_category,bio_tr,rest_index, amount_fractions,amount_flows,dest_fr,company_level);
[PI] = energy_indicators( PI,amount_fractions,amount_flows,energy);

%save result to sensitiv struct
sensitiv(k).TC=text(k+1,1);
sensitiv(k).fr=text(k+1,5);
sensitiv(k).col_fr=((PI(2).eff_fr(sens(k,3))-KPI(2).eff_fr(sens(k,3)))/(KPI(2).eff_fr(sens(k,3))))/sens(k,5);
sensitiv(k).col_tot=((PI(2).eff_tot-KPI(2).eff_tot)/(KPI(2).eff_tot))/sens(k,5);
sensitiv(k).sort_fr = ((PI(3).eff_fr(sens(k,3))-KPI(3).eff_fr(sens(k,3)))/(KPI(3).eff_fr(sens(k,3))))/sens(k,5);
sensitiv(k).sort_tot = ((PI(3).eff_tot-KPI(3).eff_tot)/(KPI(3).eff_tot))/sens(k,5);
sensitiv(k).recyc_fr = ((PI(4).eff_fr(sens(k,3))-KPI(4).eff_fr(sens(k,3)))/(KPI(4).eff_fr(sens(k,3))))/sens(k,5);
sensitiv(k).recyc_tot = ((PI(4).eff_tot-KPI(4).eff_tot)/(KPI(4).eff_tot))/sens(k,5);
sensitiv(k).energy = ((PI(7).eff_tot-KPI(7).eff_tot)/(KPI(7).eff_tot))/sens(k,5);
sensitiv(k).transport = ((PI(7).matrix(2).total-KPI(7).matrix(2).total)/(KPI(7).matrix(2).total))/sens(k,5);
sensitiv(k).process = ((PI(7).matrix(3).total-KPI(7).matrix(3).total)/(KPI(7).matrix(3).total))/sens(k,5);
sensitiv(k).out = ((PI(7).matrix(4).total-KPI(7).matrix(4).total)/(KPI(7).matrix(4).total))/sens(k,5);
sensitiv(k).inc = ((PI(7).matrix(4).matrix(1,2)-KPI(7).matrix(4).matrix(1,2))/(KPI(7).matrix(4).matrix(1,2)))/sens(k,5);
sensitiv(k).ad = ((PI(7).matrix(4).matrix(2,2)-KPI(7).matrix(4).matrix(2,2))/(KPI(7).matrix(4).matrix(2,2)))/sens(k,5);

%Create the PI_sens struct with the KPI struct of each iteration
PI_sens(k).TC = text(k+1,1); 
PI_sens(k).PI = PI;
iteration = ['Iteration ',num2str(k),': done'];
disp(iteration);

end
write(filename, export_filename, File,sensitiv);
else
    disp('No sensitivity data was given in Excel');
    sensitiv = [];
    PI_sens = [];
end
end

function [] = write (filename, export_filename,File,sensitiv)
[~,txt,~]= xlsread(filename,'Output','U4:AH19');

Excel = actxserver ('Excel.Application');
File = [File,'\',export_filename,];

if ~exist(File,'file')
    ExcelWorkbook = Excel.workbooks.Add;
    ExcelWorkbook.SaveAs(File,1);
    ExcelWorkbook.Close(false);
end
invoke(Excel.Workbooks,'Open',File);

xlswrite1(export_filename,cellstr('TC'),'Sensitivity',char(txt(1,1)));
xlswrite1(export_filename,cellstr('Fraction'),'Sensitivity',char(txt(1,2)));
xlswrite1(export_filename,cellstr('Coll. fr.'),'Sensitivity',char(txt(1,3)));
xlswrite1(export_filename,cellstr('Coll. tot.'),'Sensitivity',char(txt(1,4)));
xlswrite1(export_filename,cellstr('Sort. fr.'),'Sensitivity',char(txt(1,5)));
xlswrite1(export_filename,cellstr('Sort. tot.'),'Sensitivity',char(txt(1,6)));
xlswrite1(export_filename,cellstr('Recycl. fr.'),'Sensitivity',char(txt(1,7)));
xlswrite1(export_filename,cellstr('Recycl. tot.'),'Sensitivity',char(txt(1,8)));
xlswrite1(export_filename,cellstr('Energy efficiency'),'Sensitivity',char(txt(1,9)));
xlswrite1(export_filename,cellstr('Transport E.'),'Sensitivity',char(txt(1,10)));
xlswrite1(export_filename,cellstr('Process E.'),'Sensitivity',char(txt(1,11)));
xlswrite1(export_filename,cellstr('Generated E.'),'Sensitivity',char(txt(1,12)));
xlswrite1(export_filename,cellstr('From Incineration'),'Sensitivity',char(txt(1,13)));
xlswrite1(export_filename,cellstr('From AD'),'Sensitivity',char(txt(1,14)));

for i=1:(size(sensitiv,2))
    xlswrite1(export_filename,sensitiv(i).TC,'Sensitivity',char(txt(i+1,1)));
    xlswrite1(export_filename,sensitiv(i).fr,'Sensitivity',char(txt(i+1,2)));
    xlswrite1(export_filename,sensitiv(i).col_fr,'Sensitivity',char(txt(i+1,3)));
    xlswrite1(export_filename,sensitiv(i).col_tot,'Sensitivity',char(txt(i+1,4)));
    xlswrite1(export_filename,sensitiv(i).sort_fr,'Sensitivity',char(txt(i+1,5)));
    xlswrite1(export_filename,sensitiv(i).sort_tot,'Sensitivity',char(txt(i+1,6)));
    xlswrite1(export_filename,sensitiv(i).recyc_fr,'Sensitivity',char(txt(i+1,7)));
    xlswrite1(export_filename,sensitiv(i).recyc_tot,'Sensitivity',char(txt(i+1,8)));
    xlswrite1(export_filename,sensitiv(i).energy,'Sensitivity',char(txt(i+1,9)));
    xlswrite1(export_filename,sensitiv(i).transport,'Sensitivity',char(txt(i+1,10)));
    xlswrite1(export_filename,sensitiv(i).process,'Sensitivity',char(txt(i+1,11)));
    xlswrite1(export_filename,sensitiv(i).out,'Sensitivity',char(txt(i+1,12)));
    xlswrite1(export_filename,sensitiv(i).inc,'Sensitivity',char(txt(i+1,13)));
    xlswrite1(export_filename,sensitiv(i).ad,'Sensitivity',char(txt(i+1,14)));
end
invoke(Excel.ActiveWorkbook,'Save');
Excel.Quit
Excel.delete
clear Excel

disp('writing sensitivity done');
end