function [energy] = Energy( output, km, E_tr, E_pr, amount_flows,txt_fuels,txt_pr_E,amount_pr,cal_val,incineration,bio_energy, process_category)
%SUMMARY The energy function handles the data from excel and saves the
%transport, feedstock, process and generated enenrgy in the "energy" struct


%% Transport & Feedstock (energy in) 

%make a struct with the energy intensities for each fuel type
E_tr(any(isnan(E_tr),2),:)=[]; %this delete rows
E_pr(any(isnan(E_pr),2),:)=[]; %this delete rows
cal_val = repmat(cal_val,1,amount_flows);
%multiply the "%" with the "kwh/tkm"
for i =1:amount_flows
    E_tr_adj(:,i) = (E_tr(:,(i-1)*2+1)./100).*E_tr(:,(i-1)*2+2);
end

% --- Handle energy data --- 

% make process inflow table (used for process enenrgy), mass table of each flow
% (transport energy), feedstock table (energy in) and feed energy
% (energy out)
process = 1;
mass = zeros(size(km,1),amount_flows); %the total mass (t) of each flow (divided over the different waste types, (sum of fractions))
process_inflow = zeros(amount_pr,amount_flows); %Total mass (t) going in each process
feed_energy = zeros(amount_pr, amount_flows); % feedstock energy of each flow going in to each process. 
inflow = zeros(1,amount_flows);
feed = zeros(1,amount_flows);
feedstock = zeros(size(cal_val,1),amount_flows); % fooestock enenrgy in the system (from 0) 

for i =1:size(output,2)
    if output(i).type =='x' 
        mass(i,:) = sum(output(i).value); % total of all fractions of all flows flow
        if output(i).to == process
            inflow = inflow + mass(i,:); %inflow for a certain flow
            feed = feed + sum(cal_val.*output(i).value,1); % calorific value of that flow
            %when next process is different than the current 
            %save and reset the values to 0
            if output(i+1).to ~= process
                feed_energy(process,:) = feed; %feedstock energy in MJ going in each process
                process_inflow(process,:) = inflow; %weight going in each process
                feed = zeros(1,amount_flows);
                inflow = zeros(1,amount_flows);
                process = process + 1; 
            end
        end
        if output(i).from == 0
            feedstock = feedstock + cal_val.*output(i).value; %in MJ
        end
        
    end
end
energy(1).type = 'Feedstock energy';
energy(1).matrix = feedstock./3.6;
energy(1).total = sum(sum(energy(1).matrix),2);

% --- Transport energy requirement ---

amount_fuels = size(txt_fuels,1);% transport fuels
amount_pr_fuels = size(txt_pr_E,1);
transport_energy = zeros(size(km,1),amount_flows);
for i = 1:amount_fuels% transport fuels
    intensity = zeros(size(km,1),amount_flows); % make one intesitiy table for each type of fuel (kwh/tkm)
    for j =1:size(km,1)
        intensity(j,:) = E_tr_adj(amount_fuels*(j-1)+i,:);
    end
    transport_fuel(i).type = txt_fuels(i);
    transport_fuel(i).intensity = intensity;
    transport_fuel(i).km = km;
    transport_fuel(i).tonn = mass;
    %Kwh transport energy per flow and fuel type
    transport_fuel(i).energy = transport_fuel(i).intensity.*transport_fuel(i).km.*transport_fuel(i).tonn; % fuel intensity*distance*weight
    transport_fuel(i).kwh = sum(transport_fuel(i).energy);
    transport_energy = transport_energy + transport_fuel(i).energy;
end

energy(2).type = 'Transport energy';
energy(2).matrix = transport_energy; % transport energy used (kwh) used to transport all the mass from each flow (#km X waste type)
energy(2).total = sum(sum(energy(2).matrix),2);
energy(2).carriers = transport_fuel;

%% Process Energy

%make process fuel stuct. 1 dimention for each fuel type
process_energy = zeros(amount_pr,amount_flows);
for i =1:amount_pr_fuels
    pr_energy = zeros(amount_pr,amount_flows);
    for j =1:amount_pr
        pr_energy(j,:) = E_pr(amount_pr_fuels*(j-1)+i,:);
    end
    process_fuel(i).type = txt_pr_E(i);
    process_fuel(i).intensity = pr_energy;
    process_fuel(i).inflow = process_inflow;
    process_fuel(i).energy = process_fuel(i).intensity.*process_fuel(i).inflow;
    process_fuel(i).kwh= sum(process_fuel(i).energy);
    process_energy = process_energy + process_fuel(i).energy;
end

energy(3).type = 'Process energy';
energy(3).matrix = process_energy; % for each process
energy(3).total = sum(sum(energy(3).matrix),2);
energy(3).carriers = process_fuel;

%% Energy Out

energy_out = zeros(size(incineration,1) + size(bio_energy,1),3);
%brennverdi = zeros(size(incineration,1),2);

% --- Incineration ---
if ~isempty(incineration)
    for i =1:size(incineration,1) % loop through incineration processes
        incineration_matrix = zeros(size(cal_val,1),amount_flows);
        for j=1:size(output,2)
            if output(j).type == 'x'
                if output(j).to == incineration(i,1) %inflow in final process (incl max not recycled) Change based on enenrgy input file
                    incineration_matrix = incineration_matrix + output(j).value;
                    incineration_detail(i).process = incineration(i,1);
                    incineration_detail(i).input = incineration_matrix;
                end
            else            
                break
            end
        end 
        energy_out(i,1) = incineration(i,1);
        energy_out(i,2)= ((sum(feed_energy(incineration(i,1),:),2))*incineration(i,2))/3.6; %feedstock enenrgy going in incineration process * efficiency (in kwh)
        incineration_detail(i).energy_out = incineration_detail(i).input.*cal_val.*incineration(i,2)/3.6; 
        mj = sum(feed_energy(incineration(i,1),:),2);
        kg = sum(process_inflow(incineration(i,1),:),2)*1000;
        energy_out(i,3) = mj/kg; %burning value in MJ/kg
    end
    energy_out_detail(1).type = 'Incineration';
    energy_out_detail(1).struct =incineration_detail;
    
end

% --- Biological ---

%biologicel => give energy markets a name? or all markets?
if ~isempty(bio_energy)
    for i =1:size(bio_energy,1)
        energy_out(i+size(incineration,1),1)= bio_energy(i,1); % which bioprocess are we looking at now?

        for j = 1:size(output,2)
            if output(j).type == 'x'
                for t = 1:size(process_category(:,3),1)
                    %from Bio process to material market (fertilizer market)
                    if and(output(j).from == bio_energy(i,1),output(j).to == process_category(t,3)) 
                        bio_market = process_category(t,3);
                        bio_detail(i).process = bio_energy(i);
                        bio_detail(i).material = output(j).value;
                        break
                    end
                end
                for t = 1:size(process_category(:,4),1)
                    %from Bio process to energy market 
                    if and(output(j).from == bio_energy(i,1),output(j).to == process_category(t,4)) 
                        energy_market = process_category(t,4);
                        bio_detail(i).process = bio_energy(i);
                        bio_detail(i).energy = output(j).value;
                        break
                    end
                end
            end 
        end
        bio_detail(i).inflow = bio_detail(i).material + bio_detail(i).energy;
        bio_detail(i).energy_out = bio_detail(i).inflow.*bio_energy(i,2)*6.5;
        %AD = process_inflow(bio_market,:)+process_inflow(energy_market,:);
        energy_out(i+size(incineration,1),2)= sum(sum(bio_detail(i).energy_out,1),2); %waste in AD * gass content* 6.5 kwh/Nm3 gass
    end
    energy_out_detail(2).type = 'Anaerobic Digestion';
    energy_out_detail(2).struct = bio_detail;
end

if or(~isempty(incineration),~isempty(bio_energy))
    energy(4).type = 'Energy_out';
    energy(4).matrix = energy_out;
    energy(4).total = sum(energy(4).matrix(:,2));
    energy(4).carriers = energy_out_detail;
    energy(5).type = 'Mass inflow';
    energy(5).matrix = process_inflow;

end

end

