function [ model,sortedtech ] = handle_data( list_X,list_T,Data_X,Data_T,amount_fractions, inhab )
%SUMMARY: This function saves all the flow and TC data from excel in the "model" struct. 
%   First, all data is converted to the same unit: flows and technologies
%   are converted to t/yr and fractions to %. This is saved under
%   "new_data_X". TC will always be in %, so nothing is changed here. Afterwards, the fractions are
%   calculated by mulitplying the flows (and technologies) (t/yr) with the
%   fraction distribution (%). This gives the mass for the fractions in
%   t/yr. This is saved in the model struct. The sortedtech struct
%   containes all the flow data that was inserted in excel. If mulitple
%   technologies where inserted, these are seperatly stored in this struct.


list_X( :, all( isnan(list_X), 1 ) ) = []; % delete NaN columns
list_X(any(isnan(list_X),2),:)=[]; %this delete rows
list_T( :, all( isnan(list_T), 1 ) ) = []; %this delete NaN columns
Data_T(any(Data_T(:,1)==3,2),:)=[]; %this delete rows where Data_T(:,3)==3 (blue rows in excel)
list_T(any(isnan(list_T),2),:)=[]; %this delete NaN rows

new_Data_X=Data_X(:,3:end); % delete first 2 columns, we just want the data
new_Data_T=Data_T(:,3:end)./100; % delete first 2 columns, we just want the data


%% Make model stucture first all flows (X) then transfer coef (T)
for i=1:size(list_X)
    model(i).type = 'x';
    model(i).from = list_X(i,1);
    model(i).to = list_X(i,2);
end
clear i;
for i=1:size(list_T)
    model(i+size(list_X,1)).type = 't';
    model(i+size(list_X,1)).from = list_T(i,1);
    model(i+size(list_X,1)).to = list_T(i,2);
end
clear i;

%% Make units equal (flow and technology in t/yr, fractions in %)
% save in new_Data_X matrix 

for i=1:size(Data_X) % i = fractions
    if mod(i-1,amount_fractions+2)==0 % if the row presents a flow
        if Data_X(i,1)==3 %=kg/cap
            new_Data_X(i,:)= new_Data_X(i,:)*inhab/1000; %make t/yr
        end
        % how big is the current flow? 
        % used to calculate the size of the flows used by each technology
        total_current_X = new_Data_X(i,:); 
        
    else
        if mod(i-1,amount_fractions+2)==1 %technogoly
            if Data_X(i,1)==3 %=kg/cap/yr
                new_Data_X(i,:)= new_Data_X(i,:)*inhab/1000; %make t/yr
            end
            if Data_X(i,1) ~= 1 % = t/yr, change into %
                 new_Data_X(i,:)= new_Data_X(i,:)./total_current_X;
            end
            if Data_X(i,1) == 1 % = % => /100
                 new_Data_X(i,:)= new_Data_X(i,:)./100;
            end
            % gives the size of the specific technology flow (t/yr)
            total_current_technology = total_current_X.*new_Data_X(i,:);
        
        else %fractions
            for j=1:size(new_Data_X,2)
                if Data_X(i,1)==3 %kg/cap
                    %devide by 0 is not possible
                    %if total current technology is 0 (or not specified),
                    %new_data_X should be 0
                    if total_current_technology(j) == 0
                        new_Data_X(i,j) = 0;
                    else
                        new_Data_X(i,j)= (new_Data_X(i,j)*inhab/1000)./total_current_technology(j); %in %
                    end
                else
                    if Data_X(i,1)==2 %t/yr
                        if total_current_technology(j) == 0
                            new_Data_X(i,j) = 0;
                        else
                            new_Data_X(i,j)= new_Data_X(i,j)./total_current_technology(j); % in %
                        end
                    else % In %
                        new_Data_X(i,j)= new_Data_X(i,j)./100;
                    end
                end
            end
            
        end
    end
    
end
clear i;
clear j;

%% Fill model and technology struct (sortedtech)

flow = zeros(amount_fractions,size(new_Data_X,2)); %(fractions X types)
techn = zeros(amount_fractions,size(new_Data_X,2)); % (fractions X types)
transfer = zeros(amount_fractions,size(new_Data_T,2)); % (fractions X types)
current_flow_X = Data_X(1,2); % first flow
current_flow_T = Data_T(1,2); % first transfer coef
techn_index=1; % index used to add a nex item to the technology struct

%Below we calculate the mass flows for each fraction in t/yr. This based on the data that
%was given (new_data_X: flows and technology in t/yr, fractions in %)and store it in the technology struct. 
%When a flow has different techonologies, we add them together
%because the algoritm used later wants that each flow has a unique name.  
%The mass flows (t/yr) for the fractions from each specific technology (fractions X type - matrix) 
%is saved in the technology struct, the total for the flow (sum of the technologies)are saved in the "model" struct
%After the calculation of the missing flows (MFA algorithm), the flows will be separated again,
%so we have the absolute amount for each technology.


for i=1:size(Data_X) % all flow data provided by user
    if Data_X(i,2)==current_flow_X  
        if mod(i-1,(2+amount_fractions))==0 %total of the flow
            given_flow = new_Data_X(i,:);% which flow do we observe now?
            flow_index=1; 
        else 
            if mod(i-1,(2+amount_fractions)) == 1 %technologies
                given_tech = new_Data_X(i,:); % which techonoly do we observe
            else %mod>2 = fractions     
                
                flow(flow_index,:) = flow(flow_index,:) + given_flow.*given_tech.*new_Data_X(i,:); % adds up all the data from each possible technology, 
                %so we can have a total for each flow ( no distinction between technologies anymore)
                techn(flow_index,:)= given_flow.*given_tech.*new_Data_X(i,:); % calculates the flows for each technology
                flow_index = flow_index+1;
                
                if mod(i,(2+amount_fractions))==0 %the last one of the fractions? add to technology struct
                    if ~isnan(techn) % NaN shound not be added to the technology struct, zeros however are added. 
                        technology(techn_index).flow = current_flow_X;
                        technology(techn_index).value = techn;
                        techn_index = techn_index + 1;
                    end
                end
              
            end
        end
    end
    % when we are ready for the next flow, or we did the last flow, add the
    % flow matrix to the model structure. 
    if or(Data_X(i,2)~= current_flow_X,i==size(Data_X,1)) %new or last flow
        % find the index of our current flow, so we can add the info to our
        % model structure. 
        for j=1:size(list_X)
            if(list_X(j,3)==current_flow_X)
                list_X_index = j;
                break
            end
        end
        % new technology info is added on the end, we also ADD the index
        % of that flow in the model struct. this way we can easily expand
        % the model struct to the output struct 
        %(see later in MFA algorithm)
        for z=1:size(technology,2)
            if technology(z).flow == current_flow_X
                technology(z).index = j;
            end
        end
        
        current_flow_X=Data_X(i,2);%new current flow
        %mod should be = 0 so:
        given_flow = new_Data_X(i,:);%+1 because of number column
        flow_index=1;
        %export flow matrix to model structure
        model(list_X_index).value = flow;
        flow = zeros(amount_fractions,size(new_Data_X,2));
        
        
    end
end
 clear i;
 clear j;
 clear z;
  
 [~, order] = sort([technology(:).index],'ascend');
 sortedtech = technology(order);

%% Make transfer matrix for each TC and add to model struct
% transfer is the fractions X type matrix with all the fraction/type
% specific TC from a TC, this will be added to the model struct
transfer_index=1;
for i=1:size(Data_T)
    if Data_T(i,2)==current_flow_T  
        transfer(transfer_index,:) = new_Data_T(i,:);
        transfer_index = transfer_index+1;
    end
    if or(Data_T(i,2)~= current_flow_T,i==size(Data_T,1)) %new or last flow
        %determine the index from the TC in list_T, to know where to add T
        %in the model struct
        for j=1:size(list_T)
            if(list_T(j,end)==current_flow_T)
                list_T_index = j;
                break
            end
        end
        current_flow_T=Data_T(i,2);% new current flow
        
        model(list_T_index+size(list_X,1)).value = transfer;
        transfer = zeros(amount_fractions,size(new_Data_T,2));
        %already add the first fraction to the transfer matrix
        transfer_index = 1;
        transfer(transfer_index,:) = new_Data_T(i,:);
        transfer_index = transfer_index+1;
    end
end

end

