function [ model, output ] = MFA_algoritm( model,amount_flows,sortedtech )
%SUMMARY The function calculates the missing material flows and stores the result in the model and output struct.
%   The output struct contains the same flows as the model struct but
%   the flows with multiple technologies are stores seperatly in the output struct
%   The difference between the two is thus that "model" stores the sum of all the technologies
%   from a specific flow whereas "output" stores the specific technology flows seperatly.
%   There are 2 main loops in the function, the first goes through the
%   different waste types (k) and the second is a while loop that stop when
%   all unknown flows from specific waste type are known. In the while loop,
%   a selection struct is made that focusses on the flows and TC of a specific process. 
%   Based on this struct, the unknown flows are calculated if possible. 
%   If not, a new selection struct is made for the next process. 
%   This continues untill all unknowns are solved. In some cases the safety
%   condition will stop the while loop. This means that the program cannot
%   calculated all the solutions given the data in excel.

%loop through the different waste types
for k=1:amount_flows
start = 0; % start from process 0
checked = []; % gathers all checked and solved processes, so no selection struct of those should be made anymore
passed_value = [];
dec_var = true; % decides when to check checked or passed_value
safety = 0;
unknown_flows_while = 1; %take the actual value or not? now just 

% While there are unknown flows, keep the algoritm running. If not, stop and go
%to next waste type k.

while unknown_flows_while ==1
    
%% Make a selection struct
    %variables
    safety = safety + 1;
    if safety > 1000
        break;
    end
    unknown_flows_while = 0;
    index_selection = 1;
    selected = struct;
    
    %loop through model struct
    for i=1:size(model,2)
        if (model(i).from == start)
            if dec_var % true? check if process is in the checked array
                if ~ismember(model(i).to,checked) % if the to-process is not yet checked, check this process
                    current_process = model(i).to;
                    % make the selection struct for the current process 
                    for c=1:size(model,2)
                        % if the from- or to-process equals the current
                        % process, add to selection struct
                        if or(model(c).from == current_process,model(c).to == current_process)
                            selected(index_selection).type = model(c).type;
                            selected(index_selection).from = model(c).from;
                            selected(index_selection).to = model(c).to;
                            if size(model(c).value,2)<k %if no data was given, % change by isempty?
                                selected(index_selection).value = [];
                            else
                                selected(index_selection).value = model(c).value(:,k);
                            end
                            index_selection = index_selection +1;
                        end
                    end
                    % break out of model for loop. We have our process and related selection matrix
                    %so we can continue with the algoritm
                    break
                end
            else %dec var false
                if ~ismember(model(i).to,passed_value)
                    % If the to process is not in the passed value array, 
                    %do the same as above
                    current_process = model(i).to;
                    for c=1:size(model,2)
                        if or(model(c).from == current_process,model(c).to == current_process)
                            selected(index_selection).type = model(c).type;
                            selected(index_selection).from = model(c).from;
                            selected(index_selection).to = model(c).to;
                            if size(model(c).value,2)<k
                                selected(index_selection).value = [];
                            else
                                selected(index_selection).value = model(c).value(:,k);
                            end
                            index_selection = index_selection +1;
                        end
                    end
                    break   
                end  % end ismember passed value
            end% end dv    
            
        end% end from process = start
    end % end loop though model struct
    clear i;
    clear c;
    
    % change the Nan values to empty matrixes
    if ~isempty(fieldnames(selected)) % if selected is not empty
        for i=1:size(selected,2)
            if isnan(selected(i).value)
                selected(i).value = [];
            end
        end
    end    
    clear i;
    
 % check if selection is empty, means process is in passed value or checked
 % matrix, continue to next process. If we had all process, start at from 0
    if isempty(fieldnames(selected))
        start = start + 1;
        max_value = []; 
        for i=1:size(model,2)
            % fill this array with all the possible process number, the max
            % value from this is the amount of processes defined
            max_value = [max_value;model(i).from];
        end
        clear i;
        % if start value is bigger than the number of processes, start is 0
        if start>  max(max_value,2)
            start = 0;
            passed_value = [];
            
        end
    else  
%% determine the indexes and amount of unknown in the selection struct 
        
        % arrays
        unknown_ind_sel_x = [];
        known_ind_sel_x = [];
        known_ind_sel_t = [];
        unknown_ind_sel_t = [];
        unknown_ind_model_x = [];
        unknown_ind_model_t = [];
        % variables
        sum_t = 0;
        sum_inflow_x = 0;
        sum_outflow_x = 0; %sum of the known outflows of the current process
        unknown_x_sel = 0; 
        known_x_sel=0; %how many known x
        known_t_sel=0;
        unknown_t_sel=0;
        amount_x = 0; %total amount of x (known + unknown), not used
        amount_t = 0; % not used
        unknown_inflow_x = 0;
        unknown_outflow_x = 0;

        %determine (un)know_ind_sel
        for a=1:size(selected,2)
            if char(selected(a).type)== 'x'
                amount_x = amount_x + 1;

                if isempty(selected(a).value) %unknown 
                    unknown_ind_sel_x = [unknown_ind_sel_x,a]; % stores the indexes from the unknown flows in the selection struct
                    unknown_x_sel = unknown_x_sel +1; % counts how many unknowns
                    if selected(a).from == current_process %outflow
                        unknown_outflow_x = unknown_outflow_x + 1; % counts inflows
                    else %inflow
                        unknown_inflow_x = unknown_inflow_x + 1; % counts outflows
                    end
                else %known or not NAn
                    known_x_sel=known_x_sel + 1;
                    known_ind_sel_x = [known_ind_sel_x,a]; % stores the (selection) indexes from the known flows
                    if selected(a).from == current_process %outflow
                        sum_outflow_x = sum_outflow_x + selected(a).value;
                    else %inflow
                        sum_inflow_x = sum_inflow_x + selected(a).value;
                    end
                end
            else % for transfer coefficients
                amount_t = amount_t + 1;
                if ~isempty(selected(a).value)%otherwise on error
                    sum_t = sum_t + selected(a).value; % add up all the known tc's from a specific process. 
                end
                % count the empty/not empty tc
                if isempty(selected(a).value)
                    unknown_t_sel = unknown_t_sel +1;
                    unknown_ind_sel_t = [unknown_ind_sel_t,a];
                else
                    known_t_sel=known_t_sel + 1;
                    known_ind_sel_t = [known_ind_sel_t,a];
                end
            end
        end
        clear a;
        %determine unknow_ind_model_x, array with the model indexes of
        %the unknowns in selected
        for i=1:size(model,2)
            for j=1:size(unknown_ind_sel_x,2)
                if and(model(i).from == selected(unknown_ind_sel_x(j)).from ,model(i).to == selected(unknown_ind_sel_x(j)).to)
                    if char(model(i).type)== 'x'
                        unknown_ind_model_x = [unknown_ind_model_x,i];
                    end
                break
                end
            end
        end
        clear i;
        clear j;
        %determine unknow_ind_model_t, array with the model index of the t
        %unknowns in selelcted
        for i=1:size(model,2)
            for j=1:size(unknown_ind_sel_t,2)
                if and(model(i).from == selected(unknown_ind_sel_t(j)).from ,model(i).to == selected(unknown_ind_sel_t(j)).to)
                    if char(model(i).type)== 't'%previous selected(unknown_ind_sel_t(j)
                        unknown_ind_model_t = [unknown_ind_model_t,i];
                    end
                break
                end
            end
        end
        clear i;
        clear j;
      
%% Calculation
        unknown_flows_if_calc = 0; % 
        %check if unknowns in selection matrix, could be changed by using
        %the unknown x (t) sel.
        for i=1:size(selected,2)
            if isempty(selected(i).value) 
                unknown_flows_if_calc = 1;
                break
            end
        end
        clear i;
        
        if unknown_flows_if_calc %=1 if there are still unknown flows/tc,

            if unknown_x_sel ==1 % only one unknown? mass balance
                for b=1:size(unknown_ind_sel_x,2) % has the same size as unknown_ind_model_x
                    if selected(unknown_ind_sel_x(b)).from == current_process %=outflow
                        %outflow = sum inflow - sum outflow
                        model(unknown_ind_model_x(b)).value(:,k) = sum_inflow_x - sum_outflow_x;

                    else %unknown_process = inflow
                        %inflow = sum outflow - sum inflow
                        model(unknown_ind_model_x(b)).value(:,k) = sum_outflow_x - sum_inflow_x;
                    end
                end
                
                if unknown_t_sel == 0 % no unknown tc's
                    checked = [checked,current_process]; % the process has no unknowns anymore
                    dec_var = true;
                    clear b;
                end
            else % none or more than one unknown flows
                if unknown_x_sel>1
                    % do we have at least one known flow and one known tc?
                    if and(known_x_sel > 0,known_t_sel>0)%amount_t>0
                        %enough data to calculate flows/coef
                        for i=1:size(known_ind_sel_x,2)%go through all the known flows
                           if selected(known_ind_sel_x(i)).to== current_process %known=inflow
                               if unknown_inflow_x == 0 %only when all inflows are known
                                   %multiply and store at correct place
                                   for j=1:size(known_ind_sel_t,2) %go through all known t
                                       if selected(known_ind_sel_t(j)).from == current_process %is it the right TC, from the current process?
                                           for a=1:size(unknown_ind_sel_x,2) % go through the unknown x
                                               if and(selected(known_ind_sel_t(j)).from ==selected(unknown_ind_sel_x(a)).from ,selected(known_ind_sel_t(j)).to==selected(unknown_ind_sel_x(a)).to) % "from" and "to" from the tc and flow should be the same
                                                    model(unknown_ind_model_x(a)).value(:,k) = sum_inflow_x .*selected(known_ind_sel_t(j)).value; %outlfow = inflow * TC
                                                    break %go to the next known t
                                               end
                                           end
                                           clear a;
                                       else % check this
                                           dec_var = false;
                                           passed_value = [passed_value,current_process];
                                       end
                                   end
                                   clear j;
                               else %if not all inflows are known, next iteration with new process
                                    dec_var = false;
                                    passed_value = [passed_value,current_process];
                               end

                           else %known flow is outflow  
                                if unknown_outflow_x == 0
                                    for j=1:size(known_ind_sel_t,2) %go through all known t
                                       if selected(known_ind_sel_t(j)).from == current_process%is it the right TC?
                                           for a=1:size(unknown_ind_sel_x,2) % go through the unknown x
                                               if and(selected(known_ind_sel_t(j)).from == selected(unknown_ind_sel_x(a)).from ,selected(known_ind_sel_t(j)).to == selected(unknown_ind_sel_x(a)).to) % find the correct unknown flow
                                                   if sum_t == 0
                                                       step1 = 0;   
                                                   else
                                                        step1 = (selected(known_ind_sel_x(i)).value./sum_t); % determine the inflow based on the known outflow and sum of tc
                                                   end
                                                   step2 = selected(known_ind_sel_t(j)).value;
                                                   model(unknown_ind_model_x(a)).value(:,k) = step2.*step1; % determine unknown outflow
                                                   break %go to the next known t
                                               end
                                           end
                                       else %not the correct TC
                                           dec_var = false;
                                           passed_value = [passed_value,current_process];
                                       end
                                       clear a;
                                   end
                                   clear j;
                                else %if not all outflows are knwon
                                    dec_var = false;
                                    passed_value = [passed_value,current_process];
                                end

                           end
                        end % go to all known flows (i) (known_ind_sel_x)
                        clear i;                 
                    else % no known flows or amount of TC =o
                        dec_var = false;
                        passed_value = [passed_value,current_process];
                    end

                else % unknown flows = 0 => calculate TC
                    for i=1:size(unknown_ind_sel_t,2) % go through all unknown tc
                        if selected(unknown_ind_sel_t(i)).from == current_process % correct tc
                           for a=1:size(known_ind_sel_x,2) % go through the known x
                                if and(selected(unknown_ind_sel_t(i)).from ==selected(known_ind_sel_x(a)).from ,selected(unknown_ind_sel_t(i)).to==selected(known_ind_sel_x(a)).to)
                                    model(unknown_ind_model_t(i)).value(:,k) = selected(known_ind_sel_x(a)).value./sum_inflow_x; %specific outflow/sum inflow
                                    % if sum_inflow_x contains a 0, model
                                    % will contain a NaN => change in 1. if
                                    % sum_inflow is 0, outflow will be to,
                                    % so TC is 100% (=1)
                                    for c=1:size(model(unknown_ind_model_t(i)).value,1)  
                                        if isnan(model(unknown_ind_model_t(i)).value(c,k))
                                            model(unknown_ind_model_t(i)).value(c,k)=1;
                                        end
                                    end
                                    clear c;
                                    break;
                                end
                           end

                        else
                            dec_var = false;
                            passed_value = [passed_value,current_process];                                                
                        end

                    end 
                    clear i;
                end % for if unknown flows > 1
            end

        else  %(no unknown flows/tc) 
            passed_value = [passed_value,current_process];
            dec_var = false;
        end  
    end% else loop
    
%% check the while condition  
    for i=1:size(model,2)
            %if all fractions have the same fractions X type dimension, there are no unknowns anymore
            if size(model(i).value,2)<amount_flows 
                unknown_flows_while = 1;
                break
            end
    end
    clear i;
end % basic algorithm while loop(%unknown flows)
end % go through the waste types k

%% add technologies (sortedtech) to model struct store as output struct
t=1; % idex in sorted tech (go thourgh sorted tech)
output_ind=1;
j=0;

while t <=(size(sortedtech,2))%will neverhappen, break will stop the loop
    
    for i=1:size(model,2)
        if sortedtech(t).index == i %the flow is given in excel so add value from sortedtech to output
            output(output_ind).type = model(i).type;
            output(output_ind).from = model(i).from;
            output(output_ind).to = model(i).to;
            output(output_ind).value = sortedtech(t).value;
            output_ind = output_ind +1;
            t = t+1;
            j =i;
            
            if t~=size(sortedtech,2)+1 % if t not bigger size sorted tech
                break
            else
                t=size(sortedtech,2);
            end
        else % flow was not given so just add the values from model to output
            if i > j
            output(output_ind).type = model(i).type;
            output(output_ind).from = model(i).from;
            output(output_ind).to = model(i).to;
            output(output_ind).value = model(i).value;
            output_ind = output_ind +1;
            end
        end
    end
    if i==size(model,2) % been through all 
        break; % stop while loop
    end
end



end

