function [KPI] = material_indicators( model,process_category,bio_tr,rest_index, amount_fractions,amount_flows,dest_fr, company_level )
%SUMMARY Calculates the collection, sorting,(company) material efficiency. The result are saved in the KPI struct 

%Total generated waste per fraction
index = 1;
if ~isempty(process_category)
    % returns the amount of waste going from process 0 to the collection
    % processes defined in excel. For each fraction (fr) and type (matrix)
    [generated_fr, generated_matrix] = generated(amount_fractions,amount_flows,model, process_category);
    KPI(index).name = 'Generated';
    KPI(index).matrix = generated_matrix;
    KPI(index).fractions = generated_fr;
end

%Collection efficiency
index = 2;
if ~isempty(process_category)
    % multiplies the generated matrix with the prefered destination for
    % each waste flow (given in excel), in matrix and fraction dimention,
    % and calculates fraction and overall efficiencies. 
    [collected_fr, collected_matrix,eff_col_fr, eff_col_tot] = collected(amount_fractions,dest_fr,rest_index, generated_fr,generated_matrix);
    KPI(index).name = 'Collected';
    KPI(index).matrix = collected_matrix;
    KPI(index).fractions = collected_fr;
    KPI(index).eff_fr =eff_col_fr;
    KPI(index).eff_tot = eff_col_tot;
end

% sorting effeciency
index = 3;
KPI(index).name = 'Sorted';
if ~isempty(process_category)
    % Calculates all the waste flows that are going to a final recycling
    % process (processes are given in excel)
    [sorting_fr, sorting_matrix,eff_sort_fr, eff_sort_tot] = sorted(amount_fractions,amount_flows,process_category,model,generated_fr,rest_index);
    KPI(index).matrix = sorting_matrix;
    KPI(index).fractions = sorting_fr;
    KPI(index).eff_fr =eff_sort_fr;
    KPI(index).eff_tot = eff_sort_tot;
end

%Material recycling efficiency
index = 4;
KPI(index).name = 'Recycled';
if ~isempty(process_category)
    %caculates the material recycling rate based on all the material flows
    %that go to material markets. Flows from AD and composting processes to
    %energy markets or the air are also included in the  efficiencies. IF
    %info about DM adjustments are given in excel, this is carried out here
    %too. 
    [recycling_fr,recycling_matrix, eff_recyc_fr, eff_recyc_tot] = recycled(amount_fractions,amount_flows,process_category,bio_tr,model,generated_fr);
    KPI(index).matrix = recycling_matrix;
    KPI(index).fractions = recycling_fr;
    KPI(index).eff_fr =eff_recyc_fr;
    KPI(index).eff_tot = eff_recyc_tot;

end


%company matrix
index = 5;
    KPI(index).name = 'Company';
if ~isempty(company_level)
    % Calculates the material recycling rate for the company. this sums all
    % the material flows going out of the companies influence area. 
    [company_fr,company_matrix,eff_company_fr,eff_company_tot] = company(amount_fractions,amount_flows,model,generated_fr,company_level); %100% organic sent to biological treatment
    KPI(index).matrix = company_matrix;
    KPI(index).fractions = company_fr;
    KPI(index).eff_fr =eff_company_fr;
    KPI(index).eff_tot = eff_company_tot;
end


end

function [generated_fr,generated_matrix] = generated(amount_fractions,amount_flows,model,process_category)
generated_fr = zeros(amount_fractions,1);
generated_matrix = zeros(amount_fractions,amount_flows);
% add all the values from the flows coming from process 0 (outside)

for i=1:size(process_category(:,1))
    for j=1:size(model,2)
        if and(model(j).from == 0 , model(j).to == process_category(i,1)) % to the "collection processes", from 0? 
            if model(j).type =='x'
                generated_matrix = generated_matrix + model(j).value;
                generated_fr = generated_fr + sum(model(j).value,2);
            end
        end
    end
end

end

function[collected_fr, collected_matrix, eff_col_fr, eff_col_tot] = collected(amount_fractions,dest_fr,rest_index,generated_fr,generated_matrix)
%collected_matrix = zeros(amount_fractions,amount_flows);
dest_fr(isnan(dest_fr))=0;
%if the required destination for some fractions is not included, nothing is
%filled in in excel so the matrix dimentions will be too small. Therefore
%we add a zero row for these fractions. 
if size(dest_fr)< amount_fractions
    difference = amount_fractions-size(dest_fr,1);
    for z=1:difference
       dest_fr(size(dest_fr,1)+1,1) = 0;
    end
end
%individual efficiencies
collected_matrix = generated_matrix.*dest_fr;
collected_fr=sum(collected_matrix,2); % we only want the correct destinations in the indicator
eff_col_fr = collected_fr./generated_fr; %individual fraction efficiency

%Overall collection efficiency
gen = generated_fr;
if ~isempty(rest_index)
    for i=size(rest_index)
        collected_fr(rest_index(i)) = 0 ;% not include these fractions in the general indicator
        gen(rest_index(i)) =0; % why also delete generated amount from this inidcator? if nothing is done, rest will be 100%? so doens't show the efforts of the technologies
    end
end
eff_col_tot = sum(collected_fr)/sum(gen); 
%eff_col_tot = sum(collected_fr)/sum(sum(dest_fr,2).*generated_fr);% Why multiply times reduced generated?
clear t;
end

function[sorting_fr, sorting_matrix,eff_sort_fr,eff_sort_tot] = sorted(amount_fractions,amount_flows,process_category,model,generated_fr,rest_index)
sorting_matrix = zeros(amount_fractions,amount_flows);
for i=1:size(process_category(:,2))%recycling              
    for j=1:size(model,2)
        if model(j).type == 'x'
            if process_category(i,2)==model(j).to%inflow in final process (incl max not recycled)
                sorting_matrix = sorting_matrix + model(j).value;
            end
        else            
            break
        end
    end
end
sorting_matrix(rest_index,:) = 0; % no rest sent for intention to recycling
sorting_fr = sum(sorting_matrix,2);
eff_sort_fr = sorting_fr./generated_fr;
eff_sort_tot = sum(sorting_fr)/sum(generated_fr);
end

function[recycling_fr,recycling_matrix, eff_recyc_fr, eff_recyc_tot] = recycled(amount_fractions,amount_flows,process_category,bio_tr,model,generated_fr)
recycling_matrix = zeros(amount_fractions,amount_flows);
% add all the flows that go to the material markets 
% given in the Excel file(D44:D49)
for i=1:size(process_category(:,3))%recycling              
    for j=1:size(model,2)
        if model(j).type == 'x'
            %to a recycling market
            if process_category(i,3)==model(j).to 
               for t=1:size(bio_tr,1) %if multiple bio treatment processes
                    if bio_tr(t,1) == model(j).from % from a bio process to a recycling market
                        if size(bio_tr(t,:),2)<4 % energy market not filled in => only composting, no energy market
                            AD_in(t).material = model(j).value;
                        else
                            for z=1:size(model,2)% can be changed if energy markets are zero
                                if model(z).type =='x'
                                    if and(model(z).from == model(j).from,model(z).to == bio_tr(t,4)) % to energy market
                                        AD_in(t).energy = model(z).value;
                                        AD_in(t).material = model(j).value;
                                    end
                                end
                            end
                        end
                        AD_in(t).adjusted= (AD_in(t).energy+AD_in(t).material).*(bio_tr(t,2)/bio_tr(t,3)); %(DM in (after sorting?) /DM out)
                        recycling_matrix = recycling_matrix + AD_in(t).adjusted;
                    else
                        recycling_matrix = recycling_matrix + model(j).value;
                    end
                end
            end
        else            
            break
        end
    end
end          
recycling_fr = sum(recycling_matrix,2);
eff_recyc_fr = recycling_fr./generated_fr;
eff_recyc_tot = sum(recycling_fr)/sum(generated_fr);
%for t=1:size(bio_tr,1)%Adjust to original value (in tonnes, not DM adjusted anymore)
    %recycling_matrix = recycling_matrix - AD_in(t).adjusted + AD_in(t).material; % gping to material recycling (fertilizer
%end
%recycling_fr = sum(recycling_matrix,2);
end 
 function[incineration_matrix] = incinerated(amount_fractions,amount_flows,model)
incineration_matrix = zeros(amount_fractions,amount_flows);            
for j=1:size(model,2)
    if model(j).type == 'x'
        if model(j).to == 8 %inflow in final process (incl max not recycled)
            incineration_matrix = incineration_matrix + model(j).value;
        end
    else            
        break
    end
end 
end

function[energy_eff,energy_overview] = energy(energy_over)    
    
    energy_overview(1,1)= energy_over(1).value;%feedstock
    energy_overview(2,1) = energy_over(2).value;%transport
    energy_overview(3,1) = energy_over(3).value;%process
    energy_eff = energy_over(4).value/sum(energy_overview(:,1));
    energy_overview(4,1) = energy_over(4).value;%out
end

function[company_fr,company_matrix,eff_company_fr,eff_company_tot] = company(amount_fractions,amount_flows,model,generated_fr,company_level)
company_matrix = zeros(amount_fractions,amount_flows);
for k=1:size(company_level,2)
    for i=1:size(company_level(:,k))%recycling              
        for j=1:size(model,2)
            if model(j).type == 'x'
                if company_level(i,k)==model(j).to %inflow in final process (incl max not recycled)
                    company_matrix(:,k) = company_matrix(:,k) + model(j).value(:,k);
                end
            else            
                break
            end
        end
    end
end
%company_matrix(rest_index,:) = 0; % no rest sent for intention to recycling
company_fr = sum(company_matrix,2);
eff_company_fr = company_fr./generated_fr;
eff_company_tot = sum(company_fr)/sum(generated_fr);
end